from time import time as _time
import math


class EorzeaTime:
    """EorzeaTime(moon, sun, hour, minute)"""

    __slots__ = '_moon', '_sun', '_hour', '_minute', '_phase', '_guardian'

    _YEAR = 33177600
    _MOON = 2764800
    _DAY = 86400
    _HOUR = 3600
    _MINUTE = 60
    _SECOND = 1
    _EORZEA_MINUTE = 60
    _EORZEA_BELL = 60
    _EORZEA_SUN = 24
    _EORZEA_MOON = 32
    _EORZEA_YEAR = 12
    _EORZEA_TIME_CONST = 3600.0 / 175.0
    _MILLISECOND_EORZEA_PER_MINUTE = (2 + 11/12) * 1000

    def __new__(cls, moon, sun, hour, minute):
        """Kappa"""
        self = object.__new__(cls)
        hour, minute = _check_time_field(hour, minute)
        moon, sun = _check_date_field(moon, sun)
        phase = _calculate_phase(sun)
        self._moon = _calculate_moon(moon)
        self._sun = sun
        self._hour = hour
        self._minute = minute
        self._phase = _check_phase_field(phase)
        self._guardian = _the_twelve(moon)
        return self

    @property
    def moon(self):
        return self._moon

    @property
    def sun(self):
        return self._sun

    @property
    def hour(self):
        return self._hour

    @property
    def minute(self):
        return self._minute

    @property
    def phase(self):
        return self._phase

    @property
    def guardian(self):
        return self._guardian

    @classmethod
    def now(cls):
        """Eorzea current time."""

        t = _time()
        return cls._fromtimestamp(t)

    @classmethod
    def _fromtimestamp(cls, t):
        et = t * cls._EORZEA_TIME_CONST
        moon = math.ceil(et / cls._MOON % cls._EORZEA_YEAR)
        sun = math.ceil(et / cls._DAY % cls._EORZEA_MOON)
        hh = int(et / cls._HOUR % cls._EORZEA_SUN)
        mm = int(et / cls._MINUTE % cls._EORZEA_BELL)
        return cls(moon, sun, hh, mm)

    @classmethod
    def weather_period(cls, step=5):
        """default step value is 5"""

        step = _check_int_field(step)

        time = _weather_period_generator(cls._weather_period_start(), step)
        return time

    @classmethod
    def _weather_period_start(cls):
        t = _time()
        et = t * cls._EORZEA_TIME_CONST
        lt = int(et / 28800) * 28800 / cls._EORZEA_TIME_CONST
        return lt

    def _cls_to_str(self):
        return "{}({}, {}, {:02d}, {:02d}, {:.2f}, {})".format(
            self.__class__.__qualname__,
            self._moon, self._sun,
            self._hour, self._minute, self._phase,
            self._guardian)

    def __repr__(self):
        return self._cls_to_str()

    def __str__(self):
        return self._cls_to_str()


def _weather_period_generator(min, step):
    n, i = 0, min
    while n < step:
        yield i
        i += 1400
        n += 1


def _the_twelve(moon):
    the_twelve = [
        "Halone",
        "Menphina",
        "Thaliak",
        "Nymeia",
        "Llymlaen",
        "Oschon",
        "Byregot",
        "Rhalgr",
        "Azeyma",
        "Nald'thal",
        "Nophica",
        "Althyk"
    ]
    return the_twelve[moon - 1]


def _calculate_moon(moon):
    th = ["First", "Second", "Third", "Fourth", "Fifth", "Sixth"]
    M_th = th[math.ceil(moon / 2) - 1]
    M_type = _astral_or_embral(moon)
    return "{} {} Moon".format(M_th, M_type)


def _astral_or_embral(moon):
    return "Astral" if moon % 2 else "Embral"


def _calculate_phase(sun):
    if sun <= 20:
        return 1 - int(abs(20 - sun) / 4) / 4
    if sun > 20:
        return int(abs(36 - sun) / 4) / 4


def _calculate_pphase(sun):
    if sun <= 20:
        return round((sun - 1) / 19, 2)
    if sun >= 20:
        return round(1 - (sun - 20) / 13, 2)


def _check_time_field(hour, minute):
    hour = _check_int_field(hour)
    minute = _check_int_field(minute)
    if not 0 <= hour <= 23:
        raise ValueError('hour must be in 0..23', hour)
    if not 0 <= minute <= 59:
        raise ValueError('minute must be in 0..59', minute)
    return hour, minute


def _check_date_field(moon, sun):
    moon = _check_int_field(moon)
    sun = _check_int_field(sun)
    if not 1 <= moon <= 12:
        raise ValueError('moon must be in 1..12', moon)
    if not 1 <= sun <= 32:
        raise ValueError('sun must be in 1..32', sun)
    return moon, sun


def _check_phase_field(phase):
    if not 0 <= phase <= 1:
        raise ValueError('phase must be in 0..1', phase)
    return phase


def _check_int_field(value):
    if not isinstance(value, int):
        raise TypeError("integer argument required")
    return value



from EorzeaEnv.EorzeaTime import EorzeaTime


class K:
    def thetime():
        while 1:
            t = str(EorzeaTime.now())
            print("\r"+t, end="")


if __name__ == "__main__":
    K.thetime()



territory = {
 'Aetherochemical Research Facility': 0,
 'Akh Afah Amphitheatre': 46,
 'Ala Mhigo': 0,
 'Alexander': 0,
 'Amdapor Keep': 28,
 'Astragalos': 0,
 'Aurum Vale': 0,
 'Azys Lla': 54,
 "Baelsar's Wall": 40,
 "Bardam's Mettle": 0,
 'Blue Sky': 0,
 'Bokairo Inn': 82,
 'Bowl of Embers': 25,
 "Brayflox's Longstop": 0,
 'Breath of the Creator': 0,
 'Carteneau Flats: Borderland Ruins': 0,
 'Castrum Abania': 0,
 'Castrum Fluminis': 0,
 'Castrum Meridianum': 0,
 'Central Azys Lla': 0,
 'Central Decks': 0,
 'Central Shroud': 3,
 'Central Thanalan': 10,
 'Chocobo Square': 0,
 'Cloud Nine': 0,
 'Coerthas Central Highlands': 21,
 'Coerthas Western Highlands': 49,
 'Command Room': 14,
 'Company Workshop - Mist': 0,
 'Company Workshop - Shirogane': 0,
 'Company Workshop - The Goblet': 0,
 'Company Workshop - The Lavender Beds': 0,
 'Containment Bay P1T6': 69,
 'Containment Bay S1T7': 66,
 'Containment Bay Z1T9': 75,
 'Copperbell Mines': 0,
 'Crystal Tower Training Grounds': 90,
 "Cutter's Cry": 0,
 "Dalamud's Shadow": 0,
 'Deltascape V1.0': 88,
 'Deltascape V2.0': 88,
 'Deltascape V3.0': 88,
 'Deltascape V4.0': 88,
 'Doma Castle': 0,
 'Dun Scaith': 58,
 'Dusk Vigil': 42,
 'Dzemael Darkhold': 0,
 'East Shroud': 4,
 'Eastern La Noscea': 18,
 'Eastern Thanalan': 11,
 'Emanation': 87,
 'Eorzean Alliance Headquarters': 0,
 'Eorzean Subterrane': 0,
 'Eureka Anemos': 91,
 'Eureka Pagos': 94,
 'Eureka Pyros': 96,
 'Eyes of the Creator': 0,
 'Flame Barracks': 0,
 'Fortemps Manor': 0,
 'Foundation': 47,
 "Frondale's Ward for Friendless Foundlings": 0,
 'Griffin Crossing': 0,
 'Halatali': 0,
 'Hall of Summoning': 0,
 'Hall of the Bestiarii': 0,
 'Haukke Manor': 0,
 'Haunted Manor': 0,
 'Heart of the Creator': 0,
 'Heart of the Sworn': 0,
 'Heaven-on-High': 0,
 "Hells' Kier": 98,
 "Hells' Lid": 0,
 'Hullbreaker Isle': 0,
 'IC-04 Main Bridge': 0,
 'IC-06 Central Decks': 0,
 'IC-06 Main Bridge': 0,
 'IC-06 Regeneration Grid': 0,
 'Idyllshire': 55,
 'Intercessory': 0,
 'Kienkan': 0,
 'Kobai Goten Apartment': 0,
 'Kobai Goten Apartment Lobby': 0,
 'Kugane': 82,
 'Kugane Castle': 0,
 'Kugane Ohashi': 0,
 'Labyrinth of the Ancients': 0,
 "Leofard's Chambers": 0,
 'Lichenweed': 73,
 'Lily Hills Apartment': 0,
 'Lily Hills Apartment Lobby': 0,
 'Limsa Lominsa': 0,
 'Limsa Lominsa Lower Decks': 15,
 'Limsa Lominsa Upper Decks': 14,
 'Lotus Stand': 0,
 'Lower Aetheroacoustic Exploratory Site': 0,
 'Lower La Noscea': 17,
 'Maelstrom Barracks': 0,
 'Main Bridge': 0,
 "Matoya's Cave": 0,
 'Middle La Noscea': 16,
 'Mist': 14,
 'Mizzenmast Inn': 0,
 'Mor Dhona': 22,
 'Mordion Gaol': 0,
 'Neverreap': 0,
 'New Gridania': 1,
 'North Shroud': 6,
 'Northern Thanalan': 13,
 'Old Gridania': 2,
 'Omega Control': 0,
 'Outer La Noscea': 24,
 'Pharos Sirius': 28,
 'Porta Decumana': 31,
 'Private Chambers - Mist': 0,
 'Private Chambers - Shirogane': 0,
 'Private Chambers - The Goblet': 0,
 'Private Chambers - The Lavender Beds': 0,
 'Private Cottage - Mist': 0,
 'Private Cottage - Shirogane': 0,
 'Private Cottage - The Goblet': 0,
 'Private Cottage - The Lavender Beds': 0,
 'Private House - Mist': 0,
 'Private House - Shirogane': 0,
 'Private House - The Goblet': 0,
 'Private House - The Lavender Beds': 0,
 'Private Mansion - Mist': 0,
 'Private Mansion - Shirogane': 0,
 'Private Mansion - The Goblet': 0,
 'Private Mansion - The Lavender Beds': 0,
 'Psiscape V1.0': 92,
 'Psiscape V2.0': 92,
 'Ragnarok Central Core': 0,
 'Ragnarok Drive Cylinder': 0,
 'Ragnarok Main Bridge': 0,
 'Reisen Temple': 0,
 'Reisen Temple Road': 0,
 "Rhalgr's Reach": 78,
 'Rhotano Sea': 0,
 'Royal Palace': 0,
 'Ruby Bazaar Offices': 0,
 'Ruling Chamber': 0,
 'Sacrificial Chamber': 0,
 "Saint Endalim's Scholasticate": 0,
 "Saint Mocianne's Arboretum": 0,
 'Sanctum of the Twelve': 0,
 'Sastasha': 0,
 'Seal Rock': 59,
 'Seat of the First Bow': 0,
 'Seat of the Lord Commander': 0,
 'Shirogane': 82,
 'Shisui of the Violet Tides': 0,
 'Sigmascape V1.0': 92,
 'Sigmascape V2.0': 92,
 'Sigmascape V3.0': 92,
 'Sigmascape V4.0': 92,
 'Singularity Reactor': 56,
 'Snowcloak': 42,
 'Sohm Al': 0,
 'Sohr Khai': 0,
 'Soul of the Creator': 0,
 'South Shroud': 5,
 'Southern Thanalan': 12,
 'Steps of Faith': 28,
 'Stone Vigil': 27,
 "Sultana's Breath Apartment": 0,
 "Sultana's Breath Apartment Lobby": 0,
 'Syrcus Tower': 0,
 'The <Emphasis>Misery</Emphasis>': 36,
 'The <Emphasis>Prima Vista</Emphasis> Bridge': 0,
 'The <Emphasis>Prima Vista</Emphasis> Tiring Room': 0,
 'The Aery': 28,
 'The Antitower': 0,
 'The Aquapolis': 0,
 'The Arm of the Father': 0,
 'The Arm of the Son': 0,
 'The Azim Steppe': 85,
 'The Battlehall': 0,
 'The Binding Coil of Bahamut': 0,
 'The Blessed Treasury': 77,
 'The Burden of the Father': 0,
 'The Burden of the Son': 0,
 'The Burn': 0,
 'The Burning Heart': 44,
 'The Chrysalis': 0,
 'The Churning Mists': 52,
 'The Cuff of the Father': 0,
 'The Cuff of the Son': 0,
 'The Diadem': 60,
 'The Doman Enclave': 84,
 'The Dravanian Forelands': 50,
 'The Dravanian Hinterlands': 51,
 'The Drowned City of Skalla': 0,
 'The Eighteenth Floor': 0,
 "The Fall of Belah'dia": 0,
 'The Feasting Grounds': 65,
 'The Fields of Glory': 67,
 'The First Altar of Djanan Qhat': 0,
 'The Fist of the Father': 0,
 'The Fist of the Son': 0,
 'The Fold': 0,
 'The Fractal Continuum': 0,
 'The Fringes': 79,
 'The Ghimlyt Dark': 0,
 'The Goblet': 33,
 'The Gold Saucer': 0,
 'The Great Gubal Library': 0,
 'The Great Hunt': 0,
 'The Holocharts': 0,
 'The Hourglass': 0,
 'The House of the Fierce': 84,
 'The Howling Eye': 26,
 'The Interdimensional Rift': 92,
 'The Jade Stoa': 93,
 'The Keeper of the Lake': 74,
 'The Lavender Beds': 34,
 'The Lightfeather Proving Grounds': 0,
 'The Limitless Blue': 28,
 'The Lochs': 81,
 'The Lost Canals of Uznair': 0,
 'The Lost City of Amdapor': 40,
 'The Manderville Tables': 0,
 'The Navel': 23,
 'The Orbonne Monastery': 0,
 'The Outer Coil': 28,
 'The Palace of the Dead': 0,
 'The Parrock': 0,
 'The Peaks': 80,
 'The Pillars': 48,
 'The Praetorium': 0,
 'The Ragnarok': 0,
 'The Resonatorium': 0,
 'The Ridorana Cataract': 0,
 'The Ridorana Lighthouse': 0,
 'The Rising Stones': 0,
 'The Roost': 0,
 'The Royal Airship Landing': 76,
 'The Royal City of Rabanastre': 0,
 'The Royal Menagerie': 0,
 'The Ruby Sea': 83,
 'The Sea of Clouds': 53,
 'The Shifting Altars of Uznair': 0,
 'The Sirensong Sea': 36,
 'The Striking Tree': 43,
 'The Sunken Temple of Qarn': 0,
 "The Swallow's Compass": 0,
 'The Tam-Tara Deepcroft': 0,
 'The Temple of the Fist': 0,
 'The Thousand Maws of Toto-Rak': 0,
 'The Vault': 0,
 'The Waking Sands': 0,
 "The Wanderer's Palace": 0,
 'The Weeping City of Mhach': 0,
 'The Weeping Saint': 21,
 'The Whorleater': 38,
 'The World of Darkness': 0,
 'The Wreath of Snakes': 99,
 'Thok ast Thok': 57,
 'Thornmarch': 30,
 'Topmast Apartment': 0,
 'Topmast Apartment Lobby': 0,
 'Transparency': 76,
 'Twin Adder Barracks': 0,
 "Ul'dah - Steps of Nald": 7,
 "Ul'dah - Steps of Thal": 8,
 'Ultimacy': 26,
 'Upper Aetheroacoustic Exploratory Site': 0,
 'Upper La Noscea': 20,
 'Void Ark': 37,
 'Western La Noscea': 19,
 'Western Thanalan': 9,
 "Wolves' Den Pier": 29,
 'Xelphatol': 40,
 'Yanxia': 84}



import unittest

from EorzeaEnv.EorzeaTime import EorzeaTime
from EorzeaEnv.EorzeaWeather import EorzeaWeather


class TestForecast (unittest.TestCase):
    def test_forecast(self):
        pagos_weather = EorzeaWeather.forecast_weather(
            "Eureka Pagos", 1542651599.999
        )
        pyros_weather = EorzeaWeather.forecast_weather(
            "Eureka Pyros", 1542591400.045
        )
        sigma_weather = EorzeaWeather.forecast_weather(
            "Sigmascape V4.0", 1542591400.045
        )
        ember_weather = EorzeaWeather.forecast_weather(
            "Bowl of Embers", 1542591400.045
        )
        self.assertEqual(pagos_weather, "Fog")
        self.assertEqual(pyros_weather, "Umbral Wind")
        self.assertEqual(sigma_weather, "Dimensional Disruption")
        self.assertEqual(ember_weather, "Heat Waves")

    def test_field(self):
        for t in (EorzeaTime.weather_period(10)):
            weather = EorzeaWeather.forecast_weather("Eureka Pyros", t)
            self.assertIsInstance(weather, str)

        with self.assertRaises(KeyError):
            pyros_weather = EorzeaWeather.forecast_weather(
                "EuPyros", 1542591400.045
            )

    def test_localize(self):
        localized_en_weather = EorzeaWeather.forecast_weather(
            "Eureka Pagos",
            1542651599.999,
            lang="en"
        )
        localized_jp_weather = EorzeaWeather.forecast_weather(
            "Eureka Pagos",
            1542651599.999,
            lang="jp"
        )
        localized_de_weather = EorzeaWeather.forecast_weather(
            "Eureka Pagos",
            1542651599.999,
            lang="de"
        )
        localized_fr_weather = EorzeaWeather.forecast_weather(
            "Eureka Pagos",
            1542651599.999,
            lang="fr"
        )
        self.assertEqual(localized_jp_weather, "霧")
        self.assertEqual(localized_fr_weather, "Brouillard")
        self.assertEqual(localized_de_weather, "Neblig")

    def test_step(self):
        timelist = [t for t in EorzeaTime.weather_period(10)]
        self.assertEqual(len(timelist), 10)

    def test_time(self):
        self.assertIsInstance(EorzeaTime.now().minute, int)
        self.assertIsInstance(EorzeaTime.now().hour, int)
        with self.assertRaises(ValueError):
            EorzeaTime(13, 1, 10, 50)
        with self.assertRaises(ValueError):
            EorzeaTime(12, 35, 10, 50)
        with self.assertRaises(ValueError):
            EorzeaTime(12, 1, 25, 50)
        with self.assertRaises(ValueError):
            EorzeaTime(12, 1, 10, 61)
        with self.assertRaises(TypeError):
            EorzeaTime("kappa", 1, 10, 50)

if __name__ == "__main__":
    unittest.main(verbosity=2)
